#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Main catalyst file.

Contains all the various variables, classes, methods and functions.
"""
from configparser import ConfigParser, ExtendedInterpolation
from typing import Any, Optional
from datetime import datetime, timedelta, timezone
from jiraone import WORK_PATH, add_log

config = ConfigParser(interpolation=ExtendedInterpolation())
files: str = f"{WORK_PATH}/ip_list.ini"
config.read(files)


def black_or_white_list(
        date_str: datetime = datetime.now(timezone.utc),
        date_format: str = "%Y-%m-%dT%H:%M:%S.%f%z",
        ip_address: dict = Any,
        section: str = None,
        action: str = "BLACKLIST"
) -> Optional[str]:
    """An IP first function, extract to black_list or white_list IPs.

    formatting - datetime.strptime(date_str, date_format)
    week = datetime.now(timezone.utc).isocalendar() - get the weeks as (year, week, weekday)
    grab = timedelta(data.day, data.second, data.microsecond, hours=data.hour, minutes=data.minute, weeks=week[1])
    format_with = datetime.now(timezone.utc).isoformat(timespec="microseconds")
    """
    data_list = ""
    if section in config:
        now = datetime.now(timezone.utc)  # get the current time.
        weeks = now.isocalendar()  # get the current time with the week and weekday
        # check if the address exist.
        if config[section]["ipv4"] == "" and config[section]["ipv6"] != "":
            # compare now and then and see if 24 hours has elapsed
            this = current_datetime_diff(config, section, now, weeks)
            if this is True:
                removed = config[section]['ipv6']
                config.remove_section(section)
                write_config(files, config)
                data_list = f"{action} {removed} removed..."
            else:
                # format %Y-%m-%dT%H:%M:%S.%f%z
                next_day = fast_forward(config, section)
                data_list = f"{action} of ipv6 still being enforced wait until " \
                            f"{next_day}"
        elif config[section]["ipv4"] != "" and config[section]["ipv6"] == "":
            this = current_datetime_diff(config, section, now, weeks)
            if this is True:
                removed = config[section]['ipv4']
                config.remove_section(section)
                write_config(files, config)
                data_list = f"{action} {removed} removed..."
            else:
                next_day = fast_forward(config, section)
                data_list = f"{action} of ipv4 still being enforced wait until " \
                            f"{next_day}"
        elif config[section]["ipv4"] == "" and config[section]["ipv6"] == "":
            # compare now and then and see if 24 hours has elapsed
            data_list = "No data available to import..."
    else:
        ip_v4 = ip_address["ipv4"]
        ip_v6 = ip_address["ipv6"]
        date = date_str.isoformat(timespec="microseconds")

        # update the new user if not exist
        implode = f"""
        [{section}]
        ipv4: {ip_v4}
        ipv6: {ip_v6}
        datetime: {date}
        """

        config.read_string(implode)
        write_config(files, config)

    return data_list


def current_datetime_diff(data, section, now, weeks) -> bool:
    """Do a comparison from the previous time to now time."""
    the_past = datetime.fromisoformat(data[section]["datetime"])  # covert our datetime string to datetime object
    then = the_past.isocalendar()  # return a tuple of the datetime object having just year, weeks, weekday
    now_time = timedelta(weeks=weeks[1], days=weeks[2], seconds=now.second, microseconds=now.microsecond,
                         hours=now.hour, minutes=now.minute)
    then_time = timedelta(weeks=then[1], days=then[2], seconds=the_past.second, microseconds=the_past.microsecond,
                          hours=the_past.hour, minutes=the_past.minute)

    diff = now_time - then_time
    if diff.total_seconds() > 3600 * 24:  # being the total seconds in a day
        mass = True
    else:
        mass = False

    return mass


def fast_forward(data, section):
    """Return the next day."""
    calc_date = datetime.fromisoformat(data[section]['datetime'])
    next_day = timedelta(days=1)
    formats = calc_date.today() + next_day

    return formats


def write_config(file, data) -> None:
    """Writes the config data back into the file."""
    with open(file, "w") as f:
        data.write(f)

    add_log(f"Writing data into {file}", "info")
